@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,6 +41,7 @@ from piquasso._backends.fock import (
     FockSimulator,
     PureFockSimulator,
 )
+from piquasso._backends.tensorflow import TensorflowPureFockSimulator
 
 from .instructions.preparations import (
     Vacuum,
@@ -108,6 +109,7 @@ __all__ = [
     "SamplingSimulator",
     "FockSimulator",
     "PureFockSimulator",
+    "TensorflowPureFockSimulator",
     # States
     "GaussianState",
     "SamplingState",
@@ -154,4 +156,4 @@ __all__ = [
     "LossyInterferometer",
 ]
 
-__version__ = "1.0.1"
+__version__ = "2.1.0"
