@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,7 +37,6 @@ def passive_linear(state: FockState, instruction: Instruction, shots: int) -> Re
         operator,
         modes=instruction.modes,
         d=state._space.d,
-        permanent_function=state._config.permanent_function,
     )
 
     state._density_matrix = (
@@ -134,7 +133,7 @@ def annihilate(state: FockState, instruction: Instruction, shots: int) -> Result
 
 
 def kerr(state: FockState, instruction: Instruction, shots: int) -> Result:
-    xi_vector = instruction._all_params["xi"]
+    xi_vector = instruction._all_params["xi_vector"]
 
     for mode_index, mode in enumerate(instruction.modes):
         xi = xi_vector[mode_index]
@@ -155,12 +154,12 @@ def kerr(state: FockState, instruction: Instruction, shots: int) -> Result:
 
 
 def cubic_phase(state: FockState, instruction: Instruction, shots: int) -> Result:
-    gamma = instruction._all_params["gamma"]
+    gamma_vector = instruction._all_params["gamma_vector"]
     hbar = state._config.hbar
 
     for index, mode in enumerate(instruction.modes):
         operator = state._space.get_single_mode_cubic_phase_operator(
-            gamma=gamma[index], hbar=hbar
+            gamma=gamma_vector[index], hbar=hbar, calculator=state._calculator
         )
         embedded_operator = state._space.embed_matrix(
             operator,
@@ -256,7 +255,6 @@ def linear(state: FockState, instruction: Instruction, shots: int) -> Result:
         modes=instruction.modes,
         passive_block=instruction._all_params["passive_block"],
         active_block=instruction._all_params["active_block"],
-        permanent_function=state._config.permanent_function,
     )
 
     state._density_matrix = (
