@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Type
+
 from .state import PureFockState
 
 from .calculations import (
@@ -33,8 +35,11 @@ from .calculations import (
 from ..calculations import attenuator
 
 from piquasso.api.simulator import Simulator
+from piquasso.api.calculator import BaseCalculator
 from piquasso.instructions import preparations, gates, measurements, channels
 
+from piquasso._backends.calculator import NumpyCalculator
+
 
 class PureFockSimulator(Simulator):
     """Performs photonic simulations using Fock representation with pure states.
@@ -64,7 +69,7 @@ class PureFockSimulator(Simulator):
         :class:`~piquasso.instructions.gates.Fourier`,
         :class:`~piquasso.instructions.gates.Kerr`,
         :class:`~piquasso.instructions.gates.CrossKerr`,
-        :class:`~piquasso.instructions.gates.CubicPhse`,
+        :class:`~piquasso.instructions.gates.CubicPhase`,
         :class:`~piquasso.instructions.gates.GaussianTransform`,
         :class:`~piquasso.instructions.gates.Squeezing`,
         :class:`~piquasso.instructions.gates.QuadraticPhase`,
@@ -79,7 +84,7 @@ class PureFockSimulator(Simulator):
         :class:`~piquasso.instructions.measurements.ParticleNumberMeasurement`.
 
     Supported channels:
-        :class:`~piquasso.instructions.channels.Attenuator`
+        :class:`~piquasso.instructions.channels.Attenuator`.
     """
 
     _state_class = PureFockState
@@ -107,3 +112,5 @@ class PureFockSimulator(Simulator):
         measurements.ParticleNumberMeasurement: particle_number_measurement,
         channels.Attenuator: attenuator,
     }
+
+    _calculator_class: Type[BaseCalculator] = NumpyCalculator
