@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Tuple, Generator, Any, Dict
+from typing import Optional, Tuple, Generator, Any, Dict
 
 import numpy as np
 
 from piquasso.api.config import Config
 from piquasso.api.exceptions import InvalidState, PiquassoException
+from piquasso.api.calculator import BaseCalculator
+
 from piquasso._math.fock import cutoff_cardinality, FockBasis
 
 from ..state import BaseFockState
@@ -38,24 +40,31 @@ class PureFockState(BaseFockState):
     where :math:`c \in \mathbb{N}` is the Fock space cutoff.
     """
 
-    def __init__(self, *, d: int, config: Config = None) -> None:
+    def __init__(
+        self, *, d: int, calculator: BaseCalculator, config: Optional[Config] = None
+    ) -> None:
         """
         Args:
-            state_vector (numpy.ndarray, optional): The initial state vector.
             d (int): The number of modes.
-            cutoff (int): The Fock space cutoff.
+            calculator (BaseCalculator): Instance containing calculation functions.
+            config (Config): Instance containing constants for the simulation.
         """
 
-        super().__init__(d=d, config=config)
+        super().__init__(d=d, calculator=calculator, config=config)
 
         self._state_vector = self._get_empty()
 
+    def _get_empty_list(self) -> list:
+        return [0.0] * self._space.cardinality
+
     def _get_empty(self) -> np.ndarray:
-        return np.zeros(shape=(self._space.cardinality,), dtype=complex)
+        return self._np.zeros(shape=(self._space.cardinality,), dtype=complex)
 
     def reset(self) -> None:
-        self._state_vector = self._get_empty()
-        self._state_vector[0] = 1.0
+        state_vector_list = self._get_empty_list()
+        state_vector_list[0] = 1.0
+
+        self._state_vector = self._np.array(state_vector_list)
 
     @property
     def nonzero_elements(self) -> Generator[Tuple[complex, FockBasis], Any, None]:
@@ -75,7 +84,7 @@ class PureFockState(BaseFockState):
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, PureFockState):
             return False
-        return np.allclose(self._state_vector, other._state_vector)
+        return self._np.allclose(self._state_vector, other._state_vector)
 
     @property
     def density_matrix(self) -> np.ndarray:
@@ -83,7 +92,7 @@ class PureFockState(BaseFockState):
 
         state_vector = self._state_vector[:cardinality]
 
-        return np.outer(state_vector, state_vector.conj())
+        return self._np.outer(state_vector, self._np.conj(state_vector))
 
     def _as_mixed(self) -> FockState:
         return FockState.from_fock_state(self)
@@ -102,7 +111,7 @@ class PureFockState(BaseFockState):
 
         index = self._space.index(occupation_number)
 
-        return np.real(
+        return self._np.real(
             self._state_vector[index].conjugate() * self._state_vector[index]
         )
 
@@ -110,22 +119,24 @@ class PureFockState(BaseFockState):
     def fock_probabilities(self) -> np.ndarray:
         cardinality = cutoff_cardinality(d=self._space.d, cutoff=self._config.cutoff)
 
-        return (self._state_vector * self._state_vector.conjugate()).real[:cardinality]
+        return self._np.real((self._state_vector * self._np.conj(self._state_vector)))[
+            :cardinality
+        ]
 
     @property
     def fock_probabilities_map(self) -> Dict[Tuple[int, ...], float]:
         probability_map: Dict[Tuple[int, ...], float] = {}
 
         for index, basis in self._space.basis:
-            probability_map[tuple(basis)] = np.abs(self._state_vector[index]) ** 2
+            probability_map[tuple(basis)] = self._np.abs(self._state_vector[index]) ** 2
 
         return probability_map
 
     def normalize(self) -> None:
-        if np.isclose(self.norm, 0):
+        if self._np.isclose(self.norm, 0):
             raise InvalidState("The norm of the state is 0.")
 
-        self._state_vector = self._state_vector / np.sqrt(self.norm)
+        self._state_vector = self._state_vector / self._np.sqrt(self.norm)
 
     def validate(self) -> None:
         """Validates the represented state.
@@ -136,7 +147,7 @@ class PureFockState(BaseFockState):
         """
         sum_of_probabilities = sum(self.fock_probabilities)
 
-        if not np.isclose(sum_of_probabilities, 1.0):
+        if not self._np.isclose(sum_of_probabilities, 1.0):
             raise InvalidState(
                 f"The sum of probabilities is {sum_of_probabilities}, "
                 "instead of 1.0:\n"
@@ -184,6 +195,8 @@ class PureFockState(BaseFockState):
             (float, float): A tuple that contains the expectation value and the
                 varianceof of the quadrature operator respectively.
         """
+        np = self._calculator.np
+
         reduced_dm = self.reduced(modes=modes).density_matrix
         annih = np.diag(np.sqrt(np.arange(1, self._config.cutoff)), 1)
         create = annih.T
@@ -195,11 +208,23 @@ class PureFockState(BaseFockState):
         else:
             rotated_quadratures = position
 
-        expctation = np.trace(np.dot(reduced_dm, rotated_quadratures)).real
+        expectation = np.real(np.trace(np.dot(reduced_dm, rotated_quadratures)))
         variance = (
-            np.trace(
-                np.dot(reduced_dm, np.dot(rotated_quadratures, rotated_quadratures))
-            ).real
-            - expctation**2
+            np.real(
+                np.trace(
+                    np.dot(reduced_dm, np.dot(rotated_quadratures, rotated_quadratures))
+                )
+            )
+            - expectation**2
         )
-        return expctation, variance
+        return expectation, variance
+
+    def mean_photon_number(self):
+        accumulator = 0.0
+
+        for index, basis in enumerate(self._space):
+            number = sum(basis)
+
+            accumulator += number * self._np.abs(self._state_vector[index]) ** 2
+
+        return accumulator
