@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,25 +13,35 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Tuple, Generator, Any, Dict, List
+from typing import Optional, Tuple, Generator, Any, Dict, List
 
 import abc
 import numpy as np
-from scipy.linalg import sqrtm
 from piquasso.api.config import Config
 from piquasso.api.state import State
+from piquasso.api.calculator import BaseCalculator
 
 from piquasso._math import fock
 from piquasso.api.exceptions import InvalidModes
 
 
 class BaseFockState(State, abc.ABC):
-    def __init__(self, *, d: int, config: Config = None) -> None:
-        super().__init__(config=config)
+    def __init__(
+        self, *, d: int, calculator: BaseCalculator, config: Optional[Config] = None
+    ) -> None:
+        super().__init__(calculator=calculator, config=config)
 
         self._space = fock.FockSpace(
             d=d,
             cutoff=self._config.cutoff,
+            calculator=calculator,
+        )
+        # NOTE: This is instantiated here, since it is costly to do so, and is needed
+        # for several, repeating calculations.
+        self._auxiliary_subspace = fock.FockSpace(
+            d=d - 1,
+            cutoff=self._config.cutoff,
+            calculator=calculator,
         )
 
     @property
@@ -47,56 +57,52 @@ class BaseFockState(State, abc.ABC):
 
     @abc.abstractmethod
     def _get_empty(self) -> np.ndarray:
-        pass
+        """Returns a full-zero representation (i.e. norm 0)."""
 
     @abc.abstractmethod
     def _as_mixed(self):
-        pass
+        """Returns the density matrix of the Fock state."""
 
     @property
     @abc.abstractmethod
     def nonzero_elements(self) -> Generator[Tuple[complex, tuple], Any, None]:
-        pass
+        """The nonzero contributions to the state representation in Fock basis."""
 
     @property
     @abc.abstractmethod
     def density_matrix(self) -> np.ndarray:
         """The density matrix of the state in terms of the Fock basis vectors."""
-        pass
 
     @abc.abstractmethod
     def reduced(self, modes: Tuple[int, ...]) -> "BaseFockState":
         """Reduces the state to a subsystem corresponding to the specified modes."""
-        pass
 
     @abc.abstractmethod
     def normalize(self) -> None:
-        pass
+        """Normalizes the state to have norm 1."""
 
     @abc.abstractmethod
     def reset(self) -> None:
         """
         Resets the Fock state to a vacuum state.
         """
-        pass
 
     @property
     @abc.abstractmethod
     def fock_probabilities_map(self) -> Dict[Tuple[int, ...], float]:
-        pass
+        """The particle number detection probabilities in a map."""
 
     @abc.abstractmethod
     def quadratures_mean_variance(
         self, modes: Tuple[int, ...], phi: float = 0
     ) -> Tuple[float, float]:
         """Calculates the mean and the variance of the quadratures of a Fock State"""
-        pass
 
     def wigner_function(
         self,
         positions: List[float],
         momentums: List[float],
-        modes: Tuple[int, ...] = None,
+        modes: Optional[Tuple[int, ...]] = None,
     ) -> np.ndarray:
         r"""
         This method calculates the Wigner function values at the specified position and
@@ -212,7 +218,7 @@ class BaseFockState(State, abc.ABC):
 
         .. math::
             \operatorname{F}(\rho_1, \rho_2) = \operatorname{Tr}(\sqrt{\sqrt{\rho_1}
-                \rho_2\sqrt{\rho_1}})^2
+                \rho_2\sqrt{\rho_1}})^2 = \operatorname{Tr}(\sqrt{\rho_1 \rho_2})^2
 
         Args:
             state: Either a :class:`~piquasso._backends.fock.pure.state.PureFockState`
@@ -222,9 +228,7 @@ class BaseFockState(State, abc.ABC):
         Returns:
             float: The calculated fidelity.
         """
-
-        sqrt_density_1 = sqrtm(self.density_matrix)
-        f = sqrtm(sqrt_density_1 @ state.density_matrix @ sqrt_density_1)
-        # Trace norm of the matrix. For more details please check:
-        # https://www.quantiki.org/wiki/trace-norm
-        return float((np.linalg.norm(f, ord="nuc") ** 2).real)
+        geometric_mean = np.sqrt(
+            np.linalg.eigvals(self.density_matrix @ state.density_matrix)
+        )
+        return np.abs(np.sum(geometric_mean)) ** 2
