@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,8 @@
 from piquasso.api.simulator import Simulator
 from piquasso.instructions import preparations, gates, measurements, channels
 
+from piquasso._backends.calculator import NumpyCalculator
+
 from .state import SamplingState
 from .calculations import (
     state_vector,
@@ -71,3 +73,5 @@ class SamplingSimulator(Simulator):
     }
 
     _state_class = SamplingState
+
+    _calculator_class = NumpyCalculator
