@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 
 from typing import Tuple
 
-import scipy
 import numpy as np
 
 from piquasso._math.symplectic import xp_symplectic_form
@@ -23,7 +22,7 @@ from piquasso._math.transformations import from_xxpp_to_xpxp_transformation_matr
 from scipy.linalg import sqrtm, schur, block_diag
 
 
-def takagi(matrix, rounding=12):
+def takagi(matrix, calculator, rounding=12):
     """Takagi factorization of complex symmetric matrices.
 
     Note:
@@ -37,14 +36,16 @@ def takagi(matrix, rounding=12):
     - https://journals.aps.org/pra/abstract/10.1103/PhysRevA.94.062109
     """
 
-    V, singular_values, W_adjoint = np.linalg.svd(matrix)
+    np = calculator.np
 
-    W = W_adjoint.conjugate().transpose()
+    V, singular_values, W_adjoint = calculator.svd(matrix)
+
+    W = np.conj(W_adjoint).T
 
     singular_value_multiplicity_map = {}
 
     for index, value in enumerate(singular_values):
-        value = np.round(value, decimals=rounding)
+        value = calculator.fallback_np.round(value, decimals=rounding)
 
         if value not in singular_value_multiplicity_map:
             singular_value_multiplicity_map[value] = [index]
@@ -56,11 +57,11 @@ def takagi(matrix, rounding=12):
     for indices in singular_value_multiplicity_map.values():
         Z = V[:, indices].transpose() @ W[:, indices]
 
-        diagonal_blocks_for_Q.append(scipy.linalg.sqrtm(Z))
+        diagonal_blocks_for_Q.append(calculator.sqrtm(Z))
 
-    Q = scipy.linalg.block_diag(*diagonal_blocks_for_Q)
+    Q = calculator.block_diag(*diagonal_blocks_for_Q)
 
-    return singular_values, V @ Q.conjugate()
+    return singular_values, V @ np.conj(Q)
 
 
 def _rotation_to_positive_above_diagonals(block_diagonal_matrix):
@@ -156,7 +157,7 @@ def williamson(matrix: np.ndarray) -> tuple:
 
     omega = xp_symplectic_form(d)
 
-    root_matrix = sqrtm(matrix)
+    root_matrix = sqrtm(matrix).real
     inverse_root_matrix = np.linalg.inv(root_matrix)
 
     block_diagonal_part, orthogonal_part = schur(
