@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,20 +14,25 @@
 # limitations under the License.
 
 import functools
-from typing import Tuple, Iterable, Generator, Any, List
+from typing import Optional, Tuple, Iterable, Generator, Any, List
 
 import numpy as np
-
 from operator import add
 
 from scipy.special import factorial, comb
-from scipy.linalg import block_diag, polar, logm, expm
 
 from piquasso._math.indices import get_operator_index
 from piquasso._math.combinatorics import partitions
 from piquasso._math.decompositions import takagi
-
-from piquasso.api.typing import PermanentFunction
+from piquasso._math.gradients import (
+    create_single_mode_displacement_gradient,
+    create_single_mode_squeezing_gradient,
+)
+from piquasso._math.gate_matrices import (
+    create_single_mode_displacement_matrix,
+    create_single_mode_squeezing_matrix,
+)
+from piquasso.api.calculator import BaseCalculator
 
 
 @functools.lru_cache()
@@ -129,14 +134,17 @@ class FockSpace(tuple):
     transformation to acquire the symmetrized tensor in the symmetrized representation.
     """
 
-    def __new__(cls, d: int, cutoff: int) -> "FockSpace":
+    def __new__(cls, d: int, cutoff: int, calculator: BaseCalculator) -> "FockSpace":
         return super().__new__(
             cls, FockBasis.create_all(d=d, cutoff=cutoff)  # type: ignore
         )
 
-    def __init__(self, *, d: int, cutoff: int) -> None:
+    def __init__(self, *, d: int, cutoff: int, calculator: BaseCalculator) -> None:
         self.d = d
         self.cutoff = cutoff
+        self.calculator = calculator
+
+        self._calculator = calculator
 
     def __deepcopy__(self, memo: Any) -> "FockSpace":
         """
@@ -155,138 +163,36 @@ class FockSpace(tuple):
         operator: np.ndarray,
         modes: Tuple[int, ...],
         d: int,
-        permanent_function: PermanentFunction,
     ) -> np.ndarray:
-        index = get_operator_index(modes)
+        indices = get_operator_index(modes)
 
-        embedded_operator = np.identity(d, dtype=complex)
+        embedded_operator = self._calculator.embed_in_identity(operator, indices, d)
 
-        embedded_operator[index] = operator
-
-        return block_diag(
+        return self._calculator.block_diag(
             *(
-                self.symmetric_tensorpower(
-                    embedded_operator, n, permanent_function=permanent_function
-                )
+                self.symmetric_tensorpower(embedded_operator, n)
                 for n in range(self.cutoff)
             )
         )
 
-    def get_single_mode_displacement_operator(
-        self,
-        *,
-        r: float,
-        phi: float,
-    ) -> np.ndarray:
-
-        r"""
-        This method generates the Displacement operator following a recursion rule.
-        Reference: https://quantum-journal.org/papers/q-2020-11-30-366/.
-
-        Args:
-        r (float): This is the Displacement amplitude. Typically this value can be
-            negative or positive depending on the desired displacement direction.
-            Note:
-                Setting :math:`|r|` to higher values will require you to have a higer
-                cuttof dimensions.
-        phi (float): This is the Dispalacement angle. Its ranges are
-            :math:`\phi \in [ 0, 2 \pi )`
-
-        Returns:
-            np.ndarray: The constructed Displacement matrix representing the Fock
-            operator.
-        """
-
-        fock_indices = np.sqrt(np.arange(self.cutoff, dtype=complex))
-        displacement = r * np.exp(1j * phi)
-
-        transformation = np.zeros((self.cutoff,) * 2, dtype=complex)
-        transformation[0, 0] = np.exp(-0.5 * r**2)
-
-        for row in range(1, self.cutoff):
-            transformation[row, 0] = (
-                displacement / fock_indices[row] * transformation[row - 1, 0]
-            )
-
-        for row in range(self.cutoff):
-            for col in range(1, self.cutoff):
-                transformation[row, col] = (
-                    -displacement.conj()
-                    / fock_indices[col]
-                    * transformation[row, col - 1]
-                ) + (
-                    fock_indices[row]
-                    / fock_indices[col]
-                    * transformation[row - 1, col - 1]
-                )
-
-        return transformation
-
     def get_single_mode_squeezing_operator(
         self,
         *,
         r: float,
         phi: float,
     ) -> np.ndarray:
-        """
-        This method generates the Squeezing operator following a recursion rule.
-        Reference: https://quantum-journal.org/papers/q-2020-11-30-366/.
-
-        Args:
-        r (float): This is the Squeezing amplitude. Typically this value can be
-            negative or positive depending on the desired squeezing direction.
-            Note:
-                Setting :math:`|r|` to higher values will require you to have a higer
-                cuttof dimensions.
-        phi (float): This is the Squeezing angle. Its ranges are
-            :math:`\phi \in [ 0, 2 \pi )`
-
-        Returns:
-            np.ndarray: The constructed Squeezing matrix representing the Fock operator.
-        """
-
-        sechr = 1.0 / np.cosh(r)
-        A = np.exp(1j * phi) * np.tanh(r)
-
-        transformation = np.zeros((self.cutoff,) * 2, dtype=complex)
-        transformation[0, 0] = np.sqrt(sechr)
-
-        fock_indices = np.sqrt(np.arange(self.cutoff, dtype=complex))
-
-        for index in range(2, self.cutoff, 2):
-            transformation[index, 0] = (
-                -fock_indices[index - 1]
-                / fock_indices[index]
-                * (transformation[index - 2, 0] * A)
+        @self.calculator.custom_gradient
+        def f(r, phi):
+            matrix = create_single_mode_squeezing_matrix(r, phi, self.cutoff)
+            grad = create_single_mode_squeezing_gradient(
+                r, phi, self.cutoff, matrix, self.calculator
             )
+            return matrix, grad
 
-        for row in range(0, self.cutoff):
-            for col in range(1, self.cutoff):
-                if (row + col) % 2 == 0:
-                    transformation[row, col] = (
-                        1
-                        / fock_indices[col]
-                        * (
-                            (
-                                fock_indices[row]
-                                * transformation[row - 1, col - 1]
-                                * sechr
-                            )
-                            + (
-                                fock_indices[col - 1]
-                                * A.conj()
-                                * transformation[row, col - 2]
-                            )
-                        )
-                    )
-
-        return transformation
+        return f(r, phi)
 
     def get_single_mode_cubic_phase_operator(
-        self,
-        *,
-        gamma: float,
-        hbar: float,
+        self, *, gamma: float, hbar: float, calculator: BaseCalculator
     ) -> np.ndarray:
 
         r"""Cubic Phase gate.
@@ -316,9 +222,11 @@ class FockSpace(tuple):
                 The resulting transformation, which could be applied to the state.
         """
 
+        np = calculator.np
+
         annih = np.diag(np.sqrt(np.arange(1, self.cutoff)), 1)
         position = (annih.T + annih) * np.sqrt(hbar / 2)
-        return expm(1j * np.linalg.matrix_power(position, 3) * (gamma / (3 * hbar)))
+        return calculator.expm(1j * calculator.powm(position, 3) * (gamma / (3 * hbar)))
 
     def embed_matrix(
         self,
@@ -326,7 +234,8 @@ class FockSpace(tuple):
         modes: Tuple[int, ...],
         auxiliary_modes: Tuple[int, ...],
     ) -> np.ndarray:
-        embedded_matrix = np.zeros((self.cardinality,) * 2, dtype=complex)
+        indices = []
+        updates = []
 
         for embedded_index, operator_basis in self.operator_basis_diagonal_on_modes(
             modes=auxiliary_modes
@@ -335,9 +244,11 @@ class FockSpace(tuple):
                 operator_basis.ket.on_modes(modes=modes),
                 operator_basis.bra.on_modes(modes=modes),
             )
-            embedded_matrix[embedded_index] = matrix[index]
 
-        return embedded_matrix
+            indices.append(embedded_index)
+            updates.append(matrix[index][0])
+
+        return self._calculator.scatter(indices, updates, dim=self.cardinality)
 
     def get_linear_fock_operator(
         self,
@@ -345,7 +256,6 @@ class FockSpace(tuple):
         modes: Tuple[int, ...],
         active_block: np.ndarray,
         passive_block: np.ndarray,
-        permanent_function: PermanentFunction,
     ) -> np.ndarray:
         r"""The matrix of the symplectic transformation in Fock space.
 
@@ -396,39 +306,40 @@ class FockSpace(tuple):
             np.ndarray:
                 The resulting transformation, which could be applied to the state.
         """
+        np = self._calculator.np
+        fallback_np = self._calculator.fallback_np
 
         d = len(modes)
         identity = np.identity(d)
         zeros = np.zeros_like(identity)
 
-        K = np.block(
+        K = self._calculator.block(
             [
                 [identity, zeros],
                 [zeros, -identity],
             ],
         )
 
-        symplectic = np.block(
+        symplectic = self._calculator.block(
             [
                 [passive_block, active_block],
-                [active_block.conj(), passive_block.conj()],
+                [np.conj(active_block), np.conj(passive_block)],
             ],
         )
 
-        U, R = polar(symplectic, side="left")
+        U, R = self._calculator.polar(symplectic, side="left")
 
-        H_active = 1j * K @ logm(R)
-        H_passive = 1j * K @ logm(U)
+        H_active = 1j * K @ self._calculator.logm(R)
+        H_passive = 1j * K @ self._calculator.logm(U)
 
-        singular_values, unitary = takagi(1j * H_active[:d, d:])
+        singular_values, unitary = takagi(1j * H_active[:d, d:], self._calculator)
 
         transformation = np.identity(self.cardinality, dtype=complex)
 
         fock_operator = self.get_passive_fock_operator(
-            unitary.conj().T @ H_passive[:d, :d],
+            np.conj(unitary).T @ H_passive[:d, :d],
             modes=modes,
             d=self.d,
-            permanent_function=permanent_function,
         )
 
         transformation = fock_operator @ transformation
@@ -442,7 +353,9 @@ class FockSpace(tuple):
             squeezing_matrix = self.embed_matrix(
                 operator,
                 modes=(mode,),
-                auxiliary_modes=tuple(np.delete(np.arange(self.d), (mode,))),
+                auxiliary_modes=tuple(
+                    fallback_np.delete(fallback_np.arange(self.d), (mode,))
+                ),
             )
 
             transformation = squeezing_matrix @ transformation
@@ -451,7 +364,6 @@ class FockSpace(tuple):
             unitary,
             modes=modes,
             d=self.d,
-            permanent_function=permanent_function,
         )
 
         transformation = fock_operator @ transformation
@@ -515,41 +427,40 @@ class FockSpace(tuple):
             )
         )
 
-    def get_subspace_basis(self, n: int, d: int = None) -> List[Tuple[int, ...]]:
+    def get_subspace_basis(
+        self, n: int, d: Optional[int] = None
+    ) -> List[Tuple[int, ...]]:
         d = d or self.d
         return FockBasis.create_on_particle_subspace(boxes=d, particles=n)
 
-    def enumerate_subspace_operator_basis(
-        self, n: int, d: int = None
-    ) -> Generator[Tuple[Tuple[int, int], FockOperatorBasis], Any, None]:
-        d = d or self.d
-        subspace_operator_basis = self.get_subspace_basis(n, d)
-
-        for index, basis in enumerate(subspace_operator_basis):
-            for dual_index, dual_basis in enumerate(subspace_operator_basis):
-                yield (index, dual_index), FockOperatorBasis(ket=basis, bra=dual_basis)
-
     def symmetric_tensorpower(
         self,
         operator: np.ndarray,
         n: int,
-        permanent_function: PermanentFunction,
     ) -> np.ndarray:
         d = len(operator)
 
-        matrix = np.empty(
-            shape=(symmetric_subspace_cardinality(d=d, n=n),) * 2,
-            dtype=complex,
-        )
+        np = self.calculator.np
 
-        for index, basis in self.enumerate_subspace_operator_basis(n, d):
-            permanent = permanent_function(operator, basis.bra, basis.ket)
+        dim = symmetric_subspace_cardinality(d=d, n=n)
 
-            matrix[index] = permanent / np.sqrt(
-                np.prod(factorial(basis.bra)) * np.prod(factorial(basis.ket))
-            )
+        matrix: List[List[complex]] = [[] for _ in range(dim)]
 
-        return matrix
+        subspace_operator_basis = self.get_subspace_basis(n, d)
+
+        for row, row_basis in enumerate(subspace_operator_basis):
+            for col_basis in subspace_operator_basis:
+                cell = self.calculator.permanent(
+                    operator,
+                    col_basis,
+                    row_basis,
+                ) / np.sqrt(
+                    np.prod(factorial(col_basis)) * np.prod(factorial(row_basis))
+                )
+
+                matrix[row].append(cell)
+
+        return np.array(matrix)
 
     def get_creation_operator(self, modes: Tuple[int, ...]) -> np.ndarray:
         operator = np.zeros(shape=(self.cardinality,) * 2, dtype=complex)
@@ -567,3 +478,14 @@ class FockSpace(tuple):
 
     def get_annihilation_operator(self, modes: Tuple[int, ...]) -> np.ndarray:
         return self.get_creation_operator(modes).transpose()
+
+    def get_single_mode_displacement_operator(self, *, r, phi):
+        @self.calculator.custom_gradient
+        def f(r, phi):
+            matrix = create_single_mode_displacement_matrix(r, phi, self.cutoff)
+            grad = create_single_mode_displacement_gradient(
+                r, phi, self.cutoff, matrix, self.calculator
+            )
+            return matrix, grad
+
+        return f(r, phi)
