@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,12 @@
 
 from typing import Tuple
 
+import functools
+
 import numpy as np
 
+from scipy.special import comb
+
 
 def get_operator_index(modes: Tuple[int, ...]) -> Tuple[np.ndarray, np.ndarray]:
     """
@@ -37,3 +41,20 @@ def get_auxiliary_operator_index(
     auxiliary_rows = tuple(np.array([modes] * len(auxiliary_modes)).transpose())
 
     return auxiliary_rows, auxiliary_modes
+
+
+@functools.lru_cache()
+def get_index_in_fock_space(element: Tuple[int, ...]) -> int:
+    return sum(
+        [
+            comb(sum(element[-i:]) + i - 1, i, exact=True)
+            for i in range(1, len(element) + 1)
+        ]
+    )
+
+
+@functools.lru_cache()
+def get_index_in_fock_subspace(element: Tuple[int, ...]) -> int:
+    return sum(
+        [comb(sum(element[-i:]) + i - 1, i, exact=True) for i in range(1, len(element))]
+    )
