@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,3 +40,7 @@ class InvalidInstruction(PiquassoException):
 
 class InvalidSimulation(PiquassoException):
     """Raised when a simulation could not be executed."""
+
+
+class NotImplementedCalculation(PiquassoException):
+    """Raiesed when a calculation is not implemented."""
