@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +14,14 @@
 # limitations under the License.
 
 import typing
-from typing import Tuple, Any, Type, Dict
+from typing import Optional, Tuple, Any, Type, Dict
 
 import numpy as np
 
 from .mode import Q
 from piquasso.core import _mixins
 from piquasso.api.exceptions import PiquassoException
+from piquasso.api.calculator import BaseCalculator
 
 if typing.TYPE_CHECKING:
     from piquasso.api.program import Program
@@ -37,7 +38,9 @@ class Instruction(_mixins.DictMixin, _mixins.RegisterMixin, _mixins.CodeMixin):
 
     _subclasses: Dict[str, Type["Instruction"]] = {}
 
-    def __init__(self, *, params: dict = None, extra_params: dict = None) -> None:
+    def __init__(
+        self, *, params: Optional[dict] = None, extra_params: Optional[dict] = None
+    ) -> None:
         self._params: dict = params or dict()
 
         self._extra_params: dict = extra_params or dict()
@@ -81,6 +84,9 @@ class Instruction(_mixins.DictMixin, _mixins.RegisterMixin, _mixins.CodeMixin):
         self.modes: Tuple[int, ...] = modes
         return self
 
+    def _postprocess(self, calculator: BaseCalculator) -> None:
+        pass
+
     def _apply_to_program_on_register(self, program: "Program", register: Q) -> None:
         program.instructions.append(self.on_modes(*register.modes))
 
