@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ from piquasso.api.result import Result
 from piquasso.api.state import State
 from piquasso.api.config import Config
 from piquasso.api.program import Program
+from piquasso.api.calculator import BaseCalculator
 from piquasso.api.exceptions import (
     InvalidParameter,
     InvalidInstruction,
@@ -41,15 +42,17 @@ class Simulator(Computer, _mixins.CodeMixin):
 
     _state_class: Type[State]
     _config_class: Type[Config] = Config
+    _calculator_class: Type[BaseCalculator] = BaseCalculator
 
     def __init__(self, d: int, config: Optional[Config] = None) -> None:
         self.d = d
         self.config = config.copy() if config is not None else self._config_class()
+        self._calculator = self._calculator_class()
 
     @property
     @abc.abstractmethod
     def _instruction_map(self) -> Dict[Type[Instruction], Callable]:
-        pass
+        """The map which associates an `Instruction` to a calculation function."""
 
     def _as_code(self):
         if self.config == Config():
@@ -71,7 +74,9 @@ class Simulator(Computer, _mixins.CodeMixin):
             State: The initial state of the simulation.
         """
 
-        return self._state_class(d=self.d, config=self.config)
+        return self._state_class(
+            d=self.d, calculator=self._calculator, config=self.config
+        )
 
     def _validate_instruction_existence(self, instructions: List[Instruction]) -> None:
         for instruction in instructions:
@@ -173,7 +178,7 @@ class Simulator(Computer, _mixins.CodeMixin):
     def execute_instructions(
         self,
         instructions: List[Instruction],
-        initial_state: State = None,
+        initial_state: Optional[State] = None,
         shots: int = 1,
     ) -> Result:
         """Executes the specified instruction list.
@@ -214,8 +219,10 @@ class Simulator(Computer, _mixins.CodeMixin):
             if not hasattr(instruction, "modes") or instruction.modes is tuple():
                 instruction.modes = tuple(range(self.d))
 
+            instruction._postprocess(self._calculator)
+
             if hasattr(instruction, "_autoscale"):
-                instruction._autoscale()  # type: ignore
+                instruction._autoscale(self._calculator)  # type: ignore
 
             calculation = self._get_calculation(instruction)
 
@@ -224,7 +231,7 @@ class Simulator(Computer, _mixins.CodeMixin):
         return result
 
     def execute(
-        self, program: Program, shots: int = 1, initial_state: State = None
+        self, program: Program, shots: int = 1, initial_state: Optional[State] = None
     ) -> Result:
         """Executes the specified program.
 
