@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import copy
 import numpy as np
 
 from piquasso.api.config import Config
+from piquasso.api.calculator import BaseCalculator
 
 
 class State(abc.ABC):
@@ -28,12 +29,23 @@ class State(abc.ABC):
 
     Properties:
         d (int): Instance attribute specifying the number of modes.
+        calculator (BaseCalculator): Instance containing calculation functions.
+        config (Config): Instance containing constants for the simulation.
     """
 
     _config_class: Type[Config] = Config
 
-    def __init__(self, config: Optional[Config] = None) -> None:
+    def __init__(
+        self,
+        calculator: BaseCalculator,
+        config: Optional[Config] = None,
+    ) -> None:
         self._config = config.copy() if config is not None else self._config_class()
+        self._calculator = calculator
+
+    @property
+    def _np(self):
+        return self._calculator.np
 
     def _get_auxiliary_modes(self, modes: Tuple[int, ...]) -> Tuple[int, ...]:
         return tuple(np.delete(np.arange(self.d), modes))
@@ -50,7 +62,7 @@ class State(abc.ABC):
     @property
     @abc.abstractmethod
     def d(self) -> int:
-        pass
+        """The number of modes."""
 
     @property
     @abc.abstractmethod
@@ -64,12 +76,10 @@ class State(abc.ABC):
         Returns:
             numpy.ndarray: The particle detection probabilities.
         """
-        pass
 
     @abc.abstractmethod
     def validate(self) -> None:
         """Validates the state."""
-        pass
 
     @abc.abstractmethod
     def get_particle_detection_probability(
@@ -87,4 +97,3 @@ class State(abc.ABC):
         Returns:
             float: The probability of detection.
         """
-        pass
