@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ import typing
 from typing import TypeVar, Generic
 
 if typing.TYPE_CHECKING:
+    from piquasso.api.calculator import BaseCalculator
     from piquasso.api.program import Program
     from piquasso.api.mode import Q
 
@@ -34,14 +35,19 @@ class DictMixin(abc.ABC):
         Args:
             dict_ (dict): The desired instance in the format of a `dict`.
         """
-        pass
 
 
 class WeightMixin(abc.ABC, Generic[_T]):
+    """Class for registering coefficients into `params`.
+
+    Most notably, it is used for specifying coefficents for `StateVector`
+    and `DensityMatrix` instances.
+    """
+
     @property
     @abc.abstractmethod
     def params(self) -> dict:
-        pass
+        """The coefficient to be multiplied must be in a `params` property."""
 
     def __mul__(self, coefficient: _T) -> "WeightMixin":
         self.params["coefficient"] *= coefficient
@@ -62,7 +68,6 @@ class RegisterMixin(abc.ABC):
             program (Program): [description]
             register (Q): [description]
         """
-        pass
 
     def copy(self) -> "RegisterMixin":
         """Copies the current object with :func:`copy.deepcopy`.
@@ -76,10 +81,15 @@ class RegisterMixin(abc.ABC):
 class CodeMixin(abc.ABC):
     @abc.abstractmethod
     def _as_code(self) -> str:
-        pass
+        """The method which returns the generated code corresponding to the instance."""
 
 
 class ScalingMixin(abc.ABC):
+    ERROR_MESSAGE_TEMPLATE = (
+        "The instruction {instruction} is not applicable to modes {modes} with the "
+        "specified parameters."
+    )
+
     @abc.abstractmethod
-    def _autoscale(self) -> None:
-        pass
+    def _autoscale(self, calculator: "BaseCalculator") -> None:
+        """The method to be executed for autoscaling."""
