@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ from piquasso._math.validations import all_in_interval
 
 from piquasso.core import _mixins
 
+from piquasso.api.calculator import BaseCalculator
 from piquasso.api.exceptions import InvalidParameter
 from piquasso.api.instruction import Gate
 
@@ -169,12 +170,12 @@ class Loss(Gate, _mixins.ScalingMixin):
             ),
         )
 
-    def _autoscale(self) -> None:
+    def _autoscale(self, calculator: BaseCalculator) -> None:
         transmissivity = self._extra_params["transmissivity"]
         if transmissivity is None or len(self.modes) == len(transmissivity):
             pass
         elif len(transmissivity) == 1:
-            self._extra_params["transmissivity"] = np.array(
+            self._extra_params["transmissivity"] = calculator.np.array(
                 [transmissivity[0]] * len(self.modes),
                 dtype=complex,
             )
