@@ -1,5 +1,5 @@
 #
-# Copyright 2021-2022 Budapest Quantum Computing Group
+# Copyright 2021-2023 Budapest Quantum Computing Group
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -45,9 +45,9 @@ Most of the gates defined here are linear gates, which can be characterized by
 import numpy as np
 
 from scipy.optimize import root_scalar
-from scipy.linalg import block_diag
 
 from piquasso.api.instruction import Gate
+from piquasso.api.calculator import BaseCalculator
 from piquasso.api.exceptions import InvalidParameter
 
 from piquasso._math.decompositions import takagi
@@ -55,16 +55,17 @@ from piquasso._math.linalg import is_square, is_symmetric, is_invertible
 from piquasso._math.symplectic import complex_symplectic_form, is_symplectic
 
 from piquasso.core import _mixins
+from typing import Optional
 
 
 class _GaussianGate(Gate):
     def __init__(
         self,
         *,
-        params: dict = None,
-        passive_block: np.ndarray = None,
-        active_block: np.ndarray = None,
-        displacement_vector: np.ndarray = None,
+        params: Optional[dict] = None,
+        passive_block: Optional[np.ndarray] = None,
+        active_block: Optional[np.ndarray] = None,
+        displacement_vector: Optional[np.ndarray] = None,
     ):
         params = params or {}
 
@@ -77,6 +78,20 @@ class _GaussianGate(Gate):
             ),
         )
 
+    def _postprocess(self, calculator):
+        np = calculator.np
+
+        passive_block = self._extra_params["passive_block"]
+        active_block = self._extra_params["active_block"]
+        displacement_vector = self._extra_params["displacement_vector"]
+
+        if passive_block is not None:
+            self._extra_params["passive_block"] = np.array(passive_block)
+        if active_block is not None:
+            self._extra_params["active_block"] = np.array(active_block)
+        if displacement_vector is not None:
+            self._extra_params["displacement_vector"] = np.array(displacement_vector)
+
 
 class _ScalableGaussianGate(
     _GaussianGate,
@@ -87,13 +102,12 @@ class _ScalableGaussianGate(
         "specified parameters."
     )
 
-    def _autoscale(self) -> None:
-
+    def _autoscale(self, calculator: BaseCalculator) -> None:
         passive_block = self._extra_params["passive_block"]
         if passive_block is None or len(self.modes) == len(passive_block):
             pass
         elif len(passive_block) == 1:
-            self._extra_params["passive_block"] = block_diag(
+            self._extra_params["passive_block"] = calculator.block_diag(
                 *[passive_block] * len(self.modes)
             )
         else:
@@ -105,7 +119,7 @@ class _ScalableGaussianGate(
         if active_block is None or len(self.modes) == len(active_block):
             pass
         elif len(active_block) == 1:
-            self._extra_params["active_block"] = block_diag(
+            self._extra_params["active_block"] = calculator.block_diag(
                 *[active_block] * len(self.modes)
             )
         else:
@@ -117,7 +131,7 @@ class _ScalableGaussianGate(
         if displacement_vector is None or len(self.modes) == len(displacement_vector):
             pass
         elif len(displacement_vector) == 1:
-            self._extra_params["displacement_vector"] = np.array(
+            self._extra_params["displacement_vector"] = calculator.np.array(
                 [displacement_vector[0]] * len(self.modes),
                 dtype=complex,
             )
@@ -204,20 +218,27 @@ class Beamsplitter(_GaussianGate):
                 (defaults to :math:`\theta=\pi/4` that gives a 50-50 beamsplitter)
         """
 
-        t = np.cos(theta)
-        r = np.exp(1j * phi) * np.sin(theta)
-
         super().__init__(
             params=dict(
                 theta=theta,
                 phi=phi,
             ),
-            passive_block=np.array(
-                [
-                    [t, -np.conj(r)],
-                    [r, t],
-                ]
-            ),
+        )
+
+    def _postprocess(self, calculator):
+        np = calculator.np
+
+        theta = self._params["theta"]
+        phi = self._params["phi"]
+
+        t = np.cos(theta)
+        r = np.exp(1j * phi) * np.sin(theta)
+
+        self._extra_params["passive_block"] = np.array(
+            [
+                [t, -np.conj(r)],
+                [r, t],
+            ]
         )
 
 
@@ -248,9 +269,16 @@ class Phaseshifter(_ScalableGaussianGate):
         """
 
         super().__init__(
-            params=dict(phi=phi), passive_block=np.diag(np.exp(1j * np.atleast_1d(phi)))
+            params=dict(phi=phi),
         )
 
+    def _postprocess(self, calculator):
+        np = calculator.np
+
+        phi = self._params["phi"]
+
+        self._extra_params["passive_block"] = np.diag(np.exp(1j * np.atleast_1d(phi)))
+
 
 class MachZehnder(_GaussianGate):
     r"""Mach-Zehnder interferometer.
@@ -410,10 +438,17 @@ class Squeezing(_ScalableGaussianGate):
         """
         super().__init__(
             params=dict(r=r, phi=phi),
-            passive_block=np.diag(np.atleast_1d(np.cosh(r))),
-            active_block=np.diag(
-                -np.atleast_1d(np.sinh(r)) * np.exp(1j * np.atleast_1d(phi))
-            ),
+        )
+
+    def _postprocess(self, calculator):
+        np = calculator.np
+
+        r = self.params["r"]
+        phi = self.params["phi"]
+
+        self._extra_params["passive_block"] = np.diag(np.atleast_1d(np.cosh(r)))
+        self._extra_params["active_block"] = np.diag(
+            -np.atleast_1d(np.sinh(r)) * np.exp(1j * np.atleast_1d(phi))
         )
 
 
@@ -472,20 +507,25 @@ class Squeezing2(_GaussianGate):
             r (float): The amplitude of the squeezing instruction.
             phi (float): The squeezing angle.
         """
-        super().__init__(
-            params=dict(r=r, phi=phi),
-            passive_block=np.array(
-                [
-                    [np.cosh(r), 0],
-                    [0, np.cosh(r)],
-                ]
-            ),
-            active_block=np.array(
-                [
-                    [0, np.sinh(r) * np.exp(1j * phi)],
-                    [np.sinh(r) * np.exp(1j * phi), 0],
-                ]
-            ),
+        super().__init__(params=dict(r=r, phi=phi))
+
+    def _postprocess(self, calculator: BaseCalculator) -> None:
+        np = calculator.np
+
+        r = self._params["r"]
+        phi = self._params["phi"]
+
+        self._extra_params["passive_block"] = np.array(
+            [
+                [np.cosh(r), 0],
+                [0, np.cosh(r)],
+            ]
+        )
+        self._extra_params["active_block"] = np.array(
+            [
+                [0, np.sinh(r) * np.exp(1j * phi)],
+                [np.sinh(r) * np.exp(1j * phi), 0],
+            ]
         )
 
 
@@ -590,7 +630,11 @@ class Displacement(_ScalableGaussianGate):
     """
 
     def __init__(
-        self, *, alpha: complex = None, r: float = None, phi: float = None
+        self,
+        *,
+        alpha: Optional[complex] = None,
+        r: Optional[float] = None,
+        phi: Optional[float] = None,
     ) -> None:
         """
         Args:
@@ -598,21 +642,32 @@ class Displacement(_ScalableGaussianGate):
             r (float): The displacement magnitude.
             phi (float): The displacement angle.
         """
-        alpha_: np.ndarray
-
         if alpha is not None and r is None and phi is None:
             params = dict(alpha=alpha)
-            alpha_ = np.atleast_1d(alpha)
         elif alpha is None and r is not None and phi is not None:
             params = dict(r=r, phi=phi)
-            alpha_ = np.atleast_1d(r) * np.exp(1j * np.atleast_1d(phi))
         else:
             raise InvalidParameter(
                 "Either specify 'alpha' only, or the combination of 'r' and 'phi': "
                 f"alpha={alpha}, r={r}, phi={phi}."
             )
 
-        super().__init__(params=params, displacement_vector=alpha_)
+        super().__init__(params=params)
+
+    def _postprocess(self, calculator):
+        np = calculator.np
+
+        displacement_vector: np.ndarray
+
+        if "alpha" in self._params:
+            alpha = self._params["alpha"]
+            displacement_vector = np.atleast_1d(alpha)
+        else:
+            r = self._params["r"]
+            phi = self._params["phi"]
+            displacement_vector = np.atleast_1d(r) * np.exp(1j * np.atleast_1d(phi))
+
+        self._extra_params["displacement_vector"] = displacement_vector
 
 
 class PositionDisplacement(_ScalableGaussianGate):
@@ -651,45 +706,7 @@ class MomentumDisplacement(_ScalableGaussianGate):
         )
 
 
-class _ScalableFockGates(Gate, _mixins.ScalingMixin):
-    ERROR_MESSAGE_TEMPLATE = (
-        "The instruction {instruction} is not applicable to modes {modes} with the "
-        "specified parameters."
-    )
-
-    def __init__(
-        self, *, params: dict = None, gamma: np.ndarray = None, xi: np.ndarray = None
-    ):
-        params = params or {}
-        super().__init__(params=params, extra_params=dict(gamma=gamma, xi=xi))
-
-    def _autoscale(self) -> None:
-        gamma = self._extra_params["gamma"]
-        xi = self._extra_params["xi"]
-        if gamma is not None and len(gamma) == len(self.modes):
-            pass
-        elif gamma is None:
-            pass
-        elif len(gamma) == 1:
-            self._extra_params["gamma"] = np.array([gamma] * len(self.modes))
-        else:
-            raise InvalidParameter(
-                self.ERROR_MESSAGE_TEMPLATE.format(instruction=self, modes=self.modes)
-            )
-
-        if xi is not None and len(xi) == len(self.modes):
-            pass
-        elif xi is None:
-            pass
-        elif len(xi) == 1:
-            self._extra_params["xi"] = np.array([xi] * len(self.modes))
-        else:
-            raise InvalidParameter(
-                self.ERROR_MESSAGE_TEMPLATE.format(instruction=self, modes=self.modes)
-            )
-
-
-class CubicPhase(_ScalableFockGates):
+class CubicPhase(Gate, _mixins.ScalingMixin):
     r"""Cubic Phase gate.
 
     The definition of the Cubic Phase gate is
@@ -720,10 +737,28 @@ class CubicPhase(_ScalableFockGates):
         Args:
             gamma (float): The Cubic Phase parameter.
         """
-        super().__init__(params=dict(gamma=gamma), gamma=np.atleast_1d(gamma))
+        super().__init__(params=dict(gamma=gamma))
+
+    def _postprocess(self, calculator: BaseCalculator) -> None:
+        self._extra_params["gamma_vector"] = calculator.np.atleast_1d(
+            self._params["gamma"]
+        )
+
+    def _autoscale(self, calculator: BaseCalculator) -> None:
+        gamma_vector = self._extra_params["gamma_vector"]
+        if gamma_vector is not None and len(gamma_vector) == len(self.modes):
+            pass
+        elif len(gamma_vector) == 1:
+            self._extra_params["gamma_vector"] = calculator.np.array(
+                [gamma_vector] * len(self.modes)
+            )
+        else:
+            raise InvalidParameter(
+                self.ERROR_MESSAGE_TEMPLATE.format(instruction=self, modes=self.modes)
+            )
 
 
-class Kerr(_ScalableFockGates):
+class Kerr(Gate, _mixins.ScalingMixin):
     r"""Kerr gate.
 
     The definition of the Kerr gate is
@@ -748,7 +783,23 @@ class Kerr(_ScalableFockGates):
         Args:
             xi (float): The magnitude of the Kerr nonlinear term.
         """
-        super().__init__(params=dict(xi=xi), xi=np.atleast_1d(xi))
+        super().__init__(params=dict(xi=xi))
+
+    def _postprocess(self, calculator: BaseCalculator) -> None:
+        self._extra_params["xi_vector"] = calculator.np.atleast_1d(self._params["xi"])
+
+    def _autoscale(self, calculator: BaseCalculator) -> None:
+        xi_vector = self._extra_params["xi_vector"]
+        if xi_vector is not None and len(xi_vector) == len(self.modes):
+            pass
+        elif len(xi_vector) == 1:
+            self._extra_params["xi_vector"] = calculator.np.array(
+                [xi_vector[0]] * len(self.modes)
+            )
+        else:
+            raise InvalidParameter(
+                self.ERROR_MESSAGE_TEMPLATE.format(instruction=self, modes=self.modes)
+            )
 
 
 class CrossKerr(Gate):
@@ -809,7 +860,18 @@ class Graph(Gate):
         if not is_symmetric(adjacency_matrix):
             raise InvalidParameter("The adjacency matrix should be symmetric.")
 
-        singular_values, unitary = takagi(adjacency_matrix)
+        super().__init__(
+            params=dict(
+                adjacency_matrix=adjacency_matrix,
+                mean_photon_number=mean_photon_number,
+            ),
+        )
+
+    def _postprocess(self, calculator: BaseCalculator) -> None:
+        adjacency_matrix = self._params["adjacency_matrix"]
+        mean_photon_number = self._params["mean_photon_number"]
+
+        singular_values, unitary = takagi(adjacency_matrix, calculator)
 
         scaling = self._get_scaling(singular_values, mean_photon_number)
 
@@ -823,16 +885,8 @@ class Graph(Gate):
 
         interferometer = Interferometer(unitary)
 
-        super().__init__(
-            params=dict(
-                adjacency_matrix=adjacency_matrix,
-                mean_photon_number=mean_photon_number,
-            ),
-            extra_params=dict(
-                squeezing=squeezing,
-                interferometer=interferometer,
-            ),
-        )
+        self._extra_params["squeezing"] = squeezing
+        self._extra_params["interferometer"] = interferometer
 
     def _get_scaling(
         self, singular_values: np.ndarray, mean_photon_number: float
